'use strict'

/**
 * Module dependencies.
 */

var integration = require('@segment/analytics.js-integration')

/**
 * Expose `Salesforce DMP` integration.
 */

var SalesforceDMP = module.exports = integration('Salesforce DMP')
  .option('confId', '')
  .option('trackFireEvents', [])
  .option('eventAttributeMap', {})
  .option('namespace', '')
  .tag('init', '<script src="//cdn.krxd.net/controltag/{{ confId }}.js">')
  .tag('usermatch', '<img src="https://beacon.krxd.net/usermatch.gif?partner=segment_io&partner_uid={{ userId }}">')

SalesforceDMP.prototype.initialize = function () {
  // We require a confid to run this integration.
  if (!this.options.confId) return
  var self = this

  window.Krux||((window.Krux=function(){window.Krux.q.push(arguments)}).q=[]) //eslint-disable-line

  this.load('init', function () {
    self.ready()
  })
}

SalesforceDMP.prototype.loaded = function () {
  return !!(window.Krux && window.Krux.q && window.Krux.ns)
}

SalesforceDMP.prototype.identify = function (identify) {
  // Identify should prefer the userId, if it exists.
  var userId = identify.userId() || identify.anonymousId()
  this.load('usermatch', {userId: userId})
}

SalesforceDMP.prototype.page = function (page) {
  // Page should prefer the anonId, if it exists.
  var userId = page.anonymousId() || page.userId()
  this.load('usermatch', {userId: userId})
}

SalesforceDMP.prototype.track = function (track) {
  // Track allows the user to re-fire the Krux pixel on specific events, as well
  // as modify the Krux datalayer, if they so choose.
  if (!this.options.namespace) return

  // Initialize kruxDataLayer
  var kruxDataLayer = window.kruxDataLayer || {}

  // Verify that this event is one we want to fire on, if a whitelist exists.

  // As a temp-fix for a Segment UI bug, strip empty strings.
  for (var i = 0; i < this.options.trackFireEvents.length; i++) {
    if (this.options.trackFireEvents[i] === '') {
      this.options.trackFireEvents.splice(i, 1)
      i--
    }
  }
  // End temp-fix

  if (this.options.trackFireEvents.length) {
    for (i = 0; i < this.options.trackFireEvents.length; i++) {
      var event = this.options.trackFireEvents[i]
      // Continue execution if we find the event in the list. Otherwise, bail out.
      if (event.toLowerCase() === track.event().toLowerCase()) break
      if (i === this.options.trackFireEvents.length - 1) return
    }
  }

  // Modify the dataLayer with any defined adjustments. It's okay if they choose to make none.
  for (var segmentProp in this.options.eventAttributeMap) {
    // { Segment Property -> Krux Property }
    var kruxKey = this.options.eventAttributeMap[segmentProp]
    var value = track.proxy('properties.' + segmentProp)
    if (value) kruxDataLayer[kruxKey] = value
  }

  // This re-fires the Krux tag with the given namespace without counting it as a new page view.
  // Namespace starts with 'ns:'
  var namespace = this.options.namespace
  if (namespace.length <= 3 || namespace.toLowerCase().slice(0, 3) !== 'ns:') namespace = 'ns:' + namespace

  // This re-fires the pixel.gif. It will silently fail if `namespace` does not match the namespace on the main Krux instance!
  window.Krux(namespace, 'page:load', function (err) { if (err) console.error(err) }, {pageView: false})
}
